@@ -8,9 +8,13 @@ from __future__ import absolute_import
 import base64
 import logging
 import os
+import shutil
 import sys
 import time
 from binascii import hexlify
+from threading import RLock
+
+from ipv8.taskmanager import TaskManager
 
 import libtorrent as lt
 
@@ -22,19 +26,17 @@ from twisted.internet.defer import CancelledError, Deferred, succeed
 from twisted.internet.task import LoopingCall
 from twisted.python.failure import Failure
 
-from Tribler.Core import NoDispersyRLock
 from Tribler.Core.DownloadConfig import DownloadConfigInterface, DownloadStartupConfig, get_default_dest_dir
 from Tribler.Core.DownloadState import DownloadState
 from Tribler.Core.Libtorrent import checkHandleAndSynchronize
 from Tribler.Core.TorrentDef import TorrentDef, TorrentDefNoMetainfo
 from Tribler.Core.Utilities import maketorrent
 from Tribler.Core.Utilities.torrent_utils import get_info_from_handle
+from Tribler.Core.Utilities.unicode import ensure_unicode
 from Tribler.Core.exceptions import SaveResumeDataError
 from Tribler.Core.osutils import fix_filebasename
-from Tribler.Core.simpledefs import DLMODE_NORMAL, DLMODE_VOD, DLSTATUS_SEEDING, DLSTATUS_STOPPED, \
-    PERSISTENTSTATE_CURRENTVERSION, dlstatus_strings
-from Tribler.pyipv8.ipv8.taskmanager import TaskManager
-
+from Tribler.Core.simpledefs import (
+    DLMODE_NORMAL, DLMODE_VOD, DLSTATUS_SEEDING, DLSTATUS_STOPPED, PERSISTENTSTATE_CURRENTVERSION, dlstatus_strings)
 
 if sys.platform == "win32":
     try:
@@ -52,7 +54,7 @@ class VODFile(object):
         self._download = d
 
         pieces = self._download.tdef.get_pieces()
-        self.pieces = [pieces[x:x + 20]for x in xrange(0, len(pieces), 20)]
+        self.pieces = [pieces[x:x + 20] for x in xrange(0, len(pieces), 20)]
         self.piecesize = self._download.tdef.get_piece_length()
 
         self.startpiece = get_info_from_handle(self._download.handle).map_file(
@@ -65,7 +67,9 @@ class VODFile(object):
 
         self._logger.debug('VODFile: get bytes %s - %s', oldpos, oldpos + args[0])
 
-        while not self._file.closed and self._download.get_byte_progress([(self._download.get_vod_fileindex(), oldpos, oldpos + args[0])]) < 1 and self._download.vod_seekpos is not None:
+        while not self._file.closed and self._download.get_byte_progress([
+            (self._download.get_vod_fileindex(), oldpos, oldpos + args[0])]) < 1 \
+                and self._download.vod_seekpos is not None:
             time.sleep(1)
 
         if self._file.closed:
@@ -95,7 +99,7 @@ class VODFile(object):
 
         self._logger.debug('VODFile: seek, get pieces %s', self._download.handle.piece_priorities())
         self._logger.debug('VODFile: seek, got pieces %s', [
-                           int(piece) for piece in self._download.handle.status().pieces])
+            int(piece) for piece in self._download.handle.status().pieces])
 
     def close(self, *args):
         self._file.close(*args)
@@ -106,7 +110,6 @@ class VODFile(object):
 
 
 class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
-
     """ Download subclass that represents a libtorrent download."""
 
     def __init__(self, session, tdef):
@@ -114,12 +117,21 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
         self._logger = logging.getLogger(self.__class__.__name__)
 
-        self.dllock = NoDispersyRLock()
+        self.dllock = RLock()
         self.session = session
         self.tdef = tdef
         self.handle = None
         self.vod_index = None
         self.orig_files = None
+        self.finished_deferred = Deferred()
+        self.finished_deferred_already_called = False
+        self.is_bootstrap_download = False
+
+        if session:
+            self.state_dir = self.session.config.get_state_dir()
+
+        # With hidden True download will not be in GET/downloads set, as a result will not be shown in GUI
+        self.hidden = False
 
         # To be able to return the progress of a stopped torrent, how far it got.
         self.progressbeforestop = 0.0
@@ -155,6 +167,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         self.deferreds_handle = []
         self.deferred_added = Deferred()
         self.deferred_removed = Deferred()
+        self.deferred_flushed = Deferred()
 
         self.handle_check_lc = self.register_task("handle_check", LoopingCall(self.check_handle))
 
@@ -195,7 +208,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         self.deferreds_handle.append(deferred)
         return deferred
 
-    def setup(self, dcfg=None, pstate=None, wrapperDelay=0, share_mode=False, checkpoint_disabled=False):
+    def setup(self, dcfg=None, pstate=None, wrapperDelay=0, share_mode=False, checkpoint_disabled=False, hidden=False):
         """
         Create a Download object. Used internally by Session.
         @param dcfg DownloadStartupConfig or None (in which case
@@ -206,16 +219,15 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         """
         # Called by any thread, assume sessionlock is held
         self.set_checkpoint_disabled(checkpoint_disabled)
-
+        self.hidden = hidden
         try:
             # The deferred to be returned
             deferred = Deferred()
             with self.dllock:
                 # Copy dlconfig, from default if not specified
-                if dcfg is None:
-                    cdcfg = DownloadStartupConfig()
-                else:
-                    cdcfg = dcfg
+                cdcfg = dcfg or DownloadStartupConfig(state_dir=self.session.config.get_state_dir())
+
+                self.is_bootstrap_download = cdcfg.is_bootstrap_download
                 self.dlconfig = cdcfg.dlconfig.copy()
                 self.dlconfig.lock = self.dllock
                 self.dlconfig.set_callback(self.dlconfig_changed_callback)
@@ -251,15 +263,15 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
             atp = {}
             atp["save_path"] = os.path.normpath(os.path.join(get_default_dest_dir(), self.get_dest_dir()))
             atp["storage_mode"] = lt.storage_mode_t.storage_mode_sparse
-            atp["paused"] = True
-            atp["auto_managed"] = False
-            atp["duplicate_is_error"] = False
             atp["hops"] = self.get_hops()
+            atp["flags"] = lt.add_torrent_params_flags_t.flag_paused | \
+                           lt.add_torrent_params_flags_t.flag_duplicate_is_error | \
+                           lt.add_torrent_params_flags_t.flag_update_subscribe
 
             if share_mode:
-                atp["flags"] = lt.add_torrent_params_flags_t.flag_share_mode
+                atp["flags"] = atp["flags"] | lt.add_torrent_params_flags_t.flag_share_mode
             if upload_mode:
-                atp["flags"] = lt.add_torrent_params_flags_t.flag_upload_mode
+                atp["flags"] = atp["flags"] | lt.add_torrent_params_flags_t.flag_upload_mode
 
             self.set_checkpoint_disabled(checkpoint_disabled)
 
@@ -285,8 +297,11 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                         self.orig_files[i] = filename_new
 
                 atp["ti"] = torrentinfo
-                has_resume_data = resume_data and isinstance(resume_data, dict)
-                if has_resume_data:
+                if resume_data and isinstance(resume_data, dict):
+                    # Rewrite save_path as a global path, if it is given as a relative path
+                    if "save_path" in resume_data and not os.path.isabs(resume_data["save_path"]):
+                        resume_data["save_path"] = six.text_type(
+                            os.path.join(self.state_dir, ensure_unicode(resume_data["save_path"], 'utf-8')))
                     atp["resume_data"] = lt.bencode(resume_data)
             else:
                 atp["url"] = self.tdef.get_url() or "magnet:?xt=urn:btih:" + hexlify(self.tdef.get_infohash())
@@ -295,7 +310,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         def on_torrent_added(handle):
             self.handle = handle
 
-            if self.handle.is_valid():
+            if self.handle and self.handle.is_valid():
                 self.set_selected_files()
 
                 user_stopped = pstate.get('download_defaults', 'user_stopped') if pstate else False
@@ -311,6 +326,11 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
                 # Limit the amount of connections if we have specified that
                 self.handle.set_max_connections(self.session.config.get_libtorrent_max_conn_download())
+
+                # Set limit on download for a bootstrap file
+                if self.is_bootstrap_download:
+                    self.handle.set_download_limit(self.session.config.get_bootstrap_max_download_rate())
+
                 return self
 
         def on_torrent_failed(failure):
@@ -395,7 +415,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
         encoded_str = ""
         for i in range(0, len(bitstr), 8):
-            encoded_str += chr(int(bitstr[i:i+8].ljust(8, '0'), 2))
+            encoded_str += chr(int(bitstr[i:i + 8].ljust(8, '0'), 2))
         return base64.b64encode(encoded_str)
 
     @checkHandleAndSynchronize(0.0)
@@ -437,7 +457,8 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     do_prio = True
             else:
                 self._logger.info(
-                    "LibtorrentDownloadImpl: could not set priority for non-existing piece %d / %d", piece, len(piecepriorities))
+                    "LibtorrentDownloadImpl: could not set priority for non-existing piece %d / %d", piece,
+                    len(piecepriorities))
         if do_prio:
             self.handle.prioritize_pieces(piecepriorities)
         else:
@@ -451,7 +472,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
             self._logger.info("LibtorrentDownloadImpl: could not get info from download handle")
 
         for fileindex, bytes_begin, bytes_end in byteranges:
-            if fileindex >= 0:
+            if fileindex >= 0 and torrent_info:
                 # Ensure the we remain within the file's boundaries
                 file_entry = torrent_info.file_at(fileindex)
                 bytes_begin = min(
@@ -478,11 +499,15 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
         alert_types = ('tracker_reply_alert', 'tracker_error_alert', 'tracker_warning_alert', 'metadata_received_alert',
                        'file_renamed_alert', 'performance_alert', 'torrent_checked_alert', 'torrent_finished_alert',
-                       'save_resume_data_alert', 'save_resume_data_failed_alert', 'state_update_alert')
+                       'save_resume_data_alert', 'save_resume_data_failed_alert', 'cache_flushed_alert')
 
         if alert_type in alert_types:
             getattr(self, 'on_' + alert_type)(alert)
 
+    def on_cache_flushed_alert(self, alert):
+        if not self.deferred_flushed.called:
+            self.deferred_flushed.callback(None)
+
     def on_save_resume_data_alert(self, alert):
         """
         Callback for the alert that contains the resume data of a specific download.
@@ -494,6 +519,14 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         resume_data = alert.resume_data
 
         self.pstate_for_restart = self.get_persistent_download_config()
+
+        # Make save_path relative if the torrent is saved in the Tribler state directory
+        if self.state_dir and 'save_path' in resume_data and os.path.abspath(resume_data['save_path']):
+            if self.state_dir == os.path.commonprefix([ensure_unicode(resume_data['save_path'], 'utf-8'),
+                                                       self.state_dir]):
+                resume_data['save_path'] = six.text_type(
+                    os.path.relpath(ensure_unicode(resume_data['save_path'], 'utf-8'), self.state_dir))
+
         self.pstate_for_restart.set('state', 'engineresumedata', resume_data)
         self._logger.debug("%s get resume data %s", hexlify(resume_data['info-hash']), resume_data)
 
@@ -573,17 +606,12 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         self.set_filepieceranges()
         self.set_selected_files()
 
-        if self.session.lm.rtorrent_handler:
-            self.session.lm.rtorrent_handler.save_torrent(self.tdef)
-        elif self.session.lm.torrent_db:
-            self.session.lm.torrent_db.addExternalTorrent(self.tdef, extra_info={'status': 'good'})
-
         self.checkpoint()
 
     def on_file_renamed_alert(self, alert):
-        unwanteddir_abs = os.path.join(self.get_save_path().decode('utf-8'), self.unwanted_directory_name)
-        if os.path.exists(unwanteddir_abs) and not os.listdir(unwanteddir_abs) and all(self.handle.file_priorities()):
-            os.rmdir(unwanteddir_abs)
+        unwanteddir_abs = os.path.join(ensure_unicode(self.get_save_path(), 'utf-8'), self.unwanted_directory_name)
+        if os.path.exists(unwanteddir_abs) and all(self.handle.file_priorities()):
+            shutil.rmtree(unwanteddir_abs, ignore_errors=True)
 
     def on_performance_alert(self, alert):
         if self.get_anon_mode() or self.ltmgr.ltsessions is None:
@@ -598,7 +626,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     "LibtorrentDownloadImpl: setting send_buffer_watermark to %s",
                     2 * settings['send_buffer_watermark'])
                 settings['send_buffer_watermark'] *= 2
-                self.ltmgr.get_session().set_settings(settings)
+                self.ltmgr.set_session_settings(self.ltmgr.get_session(), settings)
         # When the write cache is too small, double the buffer size to a maximum
         # of 64MiB. Again, this is the same mechanism as Deluge uses.
         elif alert.message().endswith("max outstanding disk writes reached"):
@@ -608,7 +636,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     "LibtorrentDownloadImpl: setting max_queued_disk_bytes to %s",
                     2 * settings['max_queued_disk_bytes'])
                 settings['max_queued_disk_bytes'] *= 2
-                self.ltmgr.get_session().set_settings(settings)
+                self.ltmgr.set_session_settings(self.ltmgr.get_session(), settings)
 
     def on_torrent_checked_alert(self, alert):
         if self.pause_after_next_hashcheck:
@@ -621,6 +649,13 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
     @checkHandleAndSynchronize()
     def on_torrent_finished_alert(self, alert):
         self.update_lt_status(self.handle.status())
+        if self.finished_deferred_already_called:
+            self._logger.warning("LibtorrentDownloadImpl: tried to repeat the call to finished_callback %s",
+                                 self.tdef.get_name())
+        else:
+            self.finished_deferred.callback(self)
+            self.finished_deferred_already_called = True
+
         progress = self.get_state().get_progress()
         if self.get_mode() == DLMODE_VOD:
             if progress == 1.0:
@@ -636,6 +671,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                         return
                     if self.get_state().get_progress() == 1.0:
                         self.set_byte_priority([(self.get_vod_fileindex(), 0, -1)], 1)
+
                 self.register_anonymous_task("reset_priorities", reactor.callLater(5, reset_priorities))
 
             if self.endbuffsize:
@@ -661,7 +697,8 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         self.correctedinfoname = fix_filebasename(self.tdef.get_name_as_unicode())
 
         # Allow correctedinfoname to be overwritten for multifile torrents only
-        if self.get_corrected_filename() and self.get_corrected_filename() != '' and 'files' in self.tdef.get_metainfo()['info']:
+        if self.get_corrected_filename() and self.get_corrected_filename() != '' and 'files' in \
+                self.tdef.get_metainfo()['info']:
             self.correctedinfoname = self.get_corrected_filename()
 
     @property
@@ -752,9 +789,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
             # self.handle.status().save_path to query the save path of a torrent. However, this attribute
             # is only included in libtorrent 1.0.9+
             status = self.handle.status()
-            if hasattr(status, 'save_path'):
-                return status.save_path
-            return self.handle.save_path()
+            return status.save_path if hasattr(status, 'save_path') else self.handle.save_path()
 
     @checkHandleAndSynchronize()
     def force_recheck(self):
@@ -805,8 +840,9 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     [(self.get_vod_fileindex(), self.vod_seekpos, self.vod_seekpos + self.prebuffsize),
                      (self.get_vod_fileindex(), -self.endbuffsize - 1, -1)], consecutive=consecutive)
             else:
-                return self.get_byte_progress([(self.get_vod_fileindex(), self.vod_seekpos, self.vod_seekpos + self.prebuffsize)],
-                                              consecutive=consecutive)
+                return self.get_byte_progress(
+                    [(self.get_vod_fileindex(), self.vod_seekpos, self.vod_seekpos + self.prebuffsize)],
+                    consecutive=consecutive)
         else:
             return 0.0
 
@@ -842,7 +878,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         peers = []
         peer_infos = self.handle.get_peer_info() if self.handle and self.handle.is_valid() else []
         for peer_info in peer_infos:
-            peer_dict = {'id': peer_info.pid.to_bytes().encode('hex'),
+            peer_dict = {'id': hexlify(peer_info.pid.to_bytes()),
                          'extended_version': peer_info.client,
                          'ip': peer_info.ip[0],
                          'port': peer_info.ip[1],
@@ -864,9 +900,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                          'have': peer_info.pieces, 'speed': peer_info.remote_dl_rate,
                          'connection_type': peer_info.connection_type,
                          'seed': bool(peer_info.flags & peer_info.seed),
-                         'upload_only': bool(peer_info.flags & peer_info.upload_only),
-                         'rstate': peer_info.read_state,
-                         'wstate': peer_info.write_state}
+                         'upload_only': bool(peer_info.flags & peer_info.upload_only)}
             peers.append(peer_dict)
         return peers
 
@@ -908,7 +942,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
         result = self.tracker_status.copy()
         result['[DHT]'] = [dht_peers, 'Working' if ltsession.is_dht_running() and public else 'Disabled']
-        result['[PeX]'] = [pex_peers, 'Working' if not self.get_anon_mode() else 'Disabled']
+        result['[PeX]'] = [pex_peers, 'Working']
         return result
 
     def set_state_callback(self, usercallback):
@@ -1033,7 +1067,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
             filename = os.path.join(self.session.get_downloads_pstate_dir(), basename)
             if not os.path.isfile(filename):
                 resume_data = self.pstate_for_restart.get('state', 'engineresumedata') \
-                              if self.pstate_for_restart else None
+                    if self.pstate_for_restart else None
 
                 # 2. If there is no saved data for this infohash, checkpoint it without data so we do not
                 #    lose it when we crash or restart before the download becomes known.
@@ -1042,7 +1076,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
                     'file-version': 1,
                     'info-hash': self.tdef.get_infohash()
                 }
-                alert = type('anonymous_alert', (object, ), dict(resume_data=resume_data))
+                alert = type('anonymous_alert', (object,), dict(resume_data=resume_data))
                 self.on_save_resume_data_alert(alert)
             return succeed(None)
 
@@ -1059,7 +1093,8 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
         pstate.set('state', 'version', PERSISTENTSTATE_CURRENTVERSION)
         if isinstance(self.tdef, TorrentDefNoMetainfo):
             pstate.set('state', 'metainfo', {
-                       'infohash': self.tdef.get_infohash(), 'name': self.tdef.get_name_as_unicode(), 'url': self.tdef.get_url()})
+                'infohash': self.tdef.get_infohash(), 'name': self.tdef.get_name_as_unicode(),
+                'url': self.tdef.get_url()})
         else:
             pstate.set('state', 'metainfo', self.tdef.get_metainfo())
 
@@ -1124,3 +1159,7 @@ class LibtorrentDownloadImpl(DownloadConfigInterface, TaskManager):
 
     def set_upload_mode(self, upload_mode):
         self.get_handle().addCallback(lambda handle: handle.set_upload_mode(upload_mode))
+
+    def force_dht_announce(self):
+        if self.handle:
+            self.handle.force_dht_announce()
