@@ -3,18 +3,19 @@ from __future__ import absolute_import
 import os
 import time
 
-from PyQt5.QtCore import QTimer, QUrl, pyqtSignal
+from PyQt5.QtCore import QTimer, QUrl, Qt, pyqtSignal
 from PyQt5.QtGui import QDesktopServices
 from PyQt5.QtWidgets import QAbstractItemView, QAction, QFileDialog, QTreeWidgetItem, QWidget
 
-from TriblerGUI.defs import BUTTON_TYPE_CONFIRM, BUTTON_TYPE_NORMAL, DLSTATUS_CIRCUITS, DLSTATUS_HASHCHECKING, \
-    DLSTATUS_METADATA, DLSTATUS_STOPPED, DLSTATUS_STOPPED_ON_ERROR, DLSTATUS_WAITING4HASHCHECK, \
-    DOWNLOADS_FILTER_ACTIVE, DOWNLOADS_FILTER_ALL, DOWNLOADS_FILTER_COMPLETED, \
-    DOWNLOADS_FILTER_CREDITMINING, DOWNLOADS_FILTER_DEFINITION, DOWNLOADS_FILTER_DOWNLOADING, DOWNLOADS_FILTER_INACTIVE
+from TriblerGUI.defs import (BUTTON_TYPE_CONFIRM, BUTTON_TYPE_NORMAL, DLSTATUS_CIRCUITS, DLSTATUS_EXIT_NODES,
+                             DLSTATUS_HASHCHECKING, DLSTATUS_METADATA, DLSTATUS_STOPPED, DLSTATUS_STOPPED_ON_ERROR,
+                             DLSTATUS_WAITING4HASHCHECK, DOWNLOADS_FILTER_ACTIVE, DOWNLOADS_FILTER_ALL,
+                             DOWNLOADS_FILTER_CHANNELS, DOWNLOADS_FILTER_COMPLETED, DOWNLOADS_FILTER_CREDITMINING,
+                             DOWNLOADS_FILTER_DEFINITION, DOWNLOADS_FILTER_DOWNLOADING, DOWNLOADS_FILTER_INACTIVE)
 from TriblerGUI.dialogs.confirmationdialog import ConfirmationDialog
 from TriblerGUI.tribler_action_menu import TriblerActionMenu
 from TriblerGUI.tribler_request_manager import TriblerRequestManager
-from TriblerGUI.utilities import format_size, format_speed
+from TriblerGUI.utilities import compose_magnetlink, format_size, format_speed
 from TriblerGUI.widgets.downloadwidgetitem import DownloadWidgetItem
 from TriblerGUI.widgets.loading_list_item import LoadingListItem
 
@@ -69,6 +70,7 @@ class DownloadsPage(QWidget):
 
         self.window().downloads_filter_input.textChanged.connect(self.on_filter_text_changed)
 
+        self.window().downloads_list.header().setSortIndicator(12, Qt.AscendingOrder)
         self.window().downloads_list.header().resizeSection(12, 146)
 
         if not self.window().vlc_available:
@@ -122,7 +124,7 @@ class DownloadsPage(QWidget):
             self.downloads_request_mgr.perform_request(url, self.on_received_downloads, priority=priority)
 
     def on_received_downloads(self, downloads):
-        if not downloads:
+        if not downloads or "downloads" not in downloads:
             return  # This might happen when closing Tribler
         loading_widget_index = self.window().downloads_list.indexOfTopLevelItem(self.loading_message_widget)
         if loading_widget_index > -1:
@@ -193,11 +195,14 @@ class DownloadsPage(QWidget):
 
             filter_match = self.window().downloads_filter_input.text().lower() in item.download_info["name"].lower()
             is_creditmining = item.download_info["credit_mining"]
+            is_channel = item.download_info["channel_download"]
             if self.filter == DOWNLOADS_FILTER_CREDITMINING:
                 item.setHidden(not is_creditmining or not filter_match)
+            elif self.filter == DOWNLOADS_FILTER_CHANNELS:
+                item.setHidden(not is_channel or not filter_match)
             else:
                 item.setHidden(not item.get_raw_download_status() in DOWNLOADS_FILTER_DEFINITION[self.filter] or \
-                               not filter_match or is_creditmining)
+                               not filter_match or is_creditmining or is_channel)
 
     def on_downloads_tab_button_clicked(self, button_name):
         if button_name == "downloads_all_button":
@@ -212,6 +217,8 @@ class DownloadsPage(QWidget):
             self.filter = DOWNLOADS_FILTER_INACTIVE
         elif button_name == "downloads_creditmining_button":
             self.filter = DOWNLOADS_FILTER_CREDITMINING
+        elif button_name == "downloads_channels_button":
+            self.filter = DOWNLOADS_FILTER_CHANNELS
 
         self.window().downloads_list.clearSelection()
         self.window().download_details_widget.hide()
@@ -285,7 +292,7 @@ class DownloadsPage(QWidget):
             infohash = selected_item.download_info["infohash"]
             self.request_mgr = TriblerRequestManager()
             self.request_mgr.perform_request("downloads/%s" % infohash, self.on_download_resumed,
-                                             method='PATCH', data="state=resume")
+                                             method='PATCH', data={"state": "resume"})
 
     def on_download_resumed(self, json_result):
         if json_result and 'modified' in json_result:
@@ -300,7 +307,7 @@ class DownloadsPage(QWidget):
             infohash = selected_item.download_info["infohash"]
             self.request_mgr = TriblerRequestManager()
             self.request_mgr.perform_request("downloads/%s" % infohash, self.on_download_stopped,
-                                             method='PATCH', data="state=stop")
+                                             method='PATCH', data={"state": "stop"})
 
     def on_play_download_clicked(self):
         self.window().left_menu_button_video_player.click()
@@ -336,7 +343,7 @@ class DownloadsPage(QWidget):
 
                 self.request_mgr = TriblerRequestManager()
                 self.request_mgr.perform_request("downloads/%s" % infohash, self.on_download_removed,
-                                                 method='DELETE', data="remove_data=%d" % action)
+                                                 method='DELETE', data={"remove_data": action})
         if self.dialog:
             self.dialog.close_dialog()
             self.dialog = None
@@ -351,7 +358,7 @@ class DownloadsPage(QWidget):
             infohash = selected_item.download_info["infohash"]
             self.request_mgr = TriblerRequestManager()
             self.request_mgr.perform_request("downloads/%s" % infohash, self.on_forced_recheck,
-                                             method='PATCH', data='state=recheck')
+                                             method='PATCH', data={"state": "recheck"})
 
     def on_forced_recheck(self, result):
         if result and "modified" in result:
@@ -366,7 +373,7 @@ class DownloadsPage(QWidget):
             infohash = selected_item.download_info["infohash"]
             self.request_mgr = TriblerRequestManager()
             self.request_mgr.perform_request("downloads/%s" % infohash, lambda _: None,
-                                             method='PATCH', data='anon_hops=%d' % hops)
+                                             method='PATCH', data={"anon_hops": hops})
 
     def on_explore_files(self):
         for selected_item in self.selected_items:
@@ -374,6 +381,31 @@ class DownloadsPage(QWidget):
                                                  selected_item.download_info["destination"]))
             QDesktopServices.openUrl(QUrl.fromLocalFile(path))
 
+    def on_move_files(self):
+        if len(self.selected_items) != 1:
+            return
+
+        dest_dir = QFileDialog.getExistingDirectory(self, "Please select the destination directory",
+                                                    self.selected_items[0].download_info["destination"],
+                                                    QFileDialog.ShowDirsOnly)
+
+        _infohash = self.selected_items[0].download_info["infohash"]
+        _name = self.selected_items[0].download_info["name"]
+
+        data = {
+            "state": "move_storage",
+            "dest_dir": dest_dir
+        }
+
+        request_mgr = TriblerRequestManager()
+        request_mgr.perform_request("downloads/%s" % _infohash,
+                                    lambda res, _, name=_name, target=dest_dir: self.on_files_moved(res, name, target),
+                                    data=data, method='PATCH')
+
+    def on_files_moved(self, response, name, dest_dir):
+        if "modified" in response and response["modified"]:
+            self.window().tray_show_message(name, "Moved to %s" % dest_dir)
+
     def on_export_download(self):
         self.export_dir = QFileDialog.getExistingDirectory(self, "Please select the destination directory", "",
                                                            QFileDialog.ShowDirsOnly)
@@ -416,6 +448,22 @@ class DownloadsPage(QWidget):
         else:
             self.window().tray_show_message("Torrent file exported", "Torrent file exported to %s" % dest_path)
 
+    def on_add_to_channel(self):
+        self.request_mgr = TriblerRequestManager()
+        for selected_item in self.selected_items:
+            infohash = selected_item.download_info["infohash"]
+            name = selected_item.download_info["name"]
+            post_data = {"uri": compose_magnetlink(infohash, name=name)}
+            self.request_mgr.perform_request("mychannel/torrents",
+                                             lambda response: self.on_added_to_channel(name, response),
+                                             method='PUT', data=post_data)
+
+    def on_added_to_channel(self, name, response):
+        if not response:
+            return
+        if 'added' in response and response['added']:
+            self.window().tray_show_message("Channel update", "%s torrent successfully added to your channel" % name)
+
     def on_right_click_item(self, pos):
         item_clicked = self.window().downloads_list.itemAt(pos)
         if not item_clicked or self.selected_items is None:
@@ -429,9 +477,11 @@ class DownloadsPage(QWidget):
         start_action = QAction('Start', self)
         stop_action = QAction('Stop', self)
         remove_download_action = QAction('Remove download', self)
+        add_to_channel_action = QAction('Add to my channel', self)
         force_recheck_action = QAction('Force recheck', self)
         export_download_action = QAction('Export .torrent file', self)
         explore_files_action = QAction('Explore files', self)
+        move_files_action = QAction('Move file storage', self)
 
         no_anon_action = QAction('No anonymity', self)
         one_hop_anon_action = QAction('One hop', self)
@@ -442,11 +492,13 @@ class DownloadsPage(QWidget):
         start_action.setEnabled(DownloadsPage.start_download_enabled(self.selected_items))
         stop_action.triggered.connect(self.on_stop_download_clicked)
         stop_action.setEnabled(DownloadsPage.stop_download_enabled(self.selected_items))
+        add_to_channel_action.triggered.connect(self.on_add_to_channel)
         remove_download_action.triggered.connect(self.on_remove_download_clicked)
         force_recheck_action.triggered.connect(self.on_force_recheck_download)
         force_recheck_action.setEnabled(DownloadsPage.force_recheck_download_enabled(self.selected_items))
         export_download_action.triggered.connect(self.on_export_download)
         explore_files_action.triggered.connect(self.on_explore_files)
+        move_files_action.triggered.connect(self.on_move_files)
 
         no_anon_action.triggered.connect(lambda: self.change_anonymity(0))
         one_hop_anon_action.triggered.connect(lambda: self.change_anonymity(1))
@@ -461,20 +513,26 @@ class DownloadsPage(QWidget):
             play_action.triggered.connect(self.on_play_download_clicked)
             menu.addAction(play_action)
         menu.addSeparator()
+        menu.addAction(add_to_channel_action)
+        menu.addSeparator()
         menu.addAction(remove_download_action)
         menu.addSeparator()
         menu.addAction(force_recheck_action)
         menu.addSeparator()
 
-        exclude_states = [DLSTATUS_METADATA, DLSTATUS_CIRCUITS, DLSTATUS_HASHCHECKING, DLSTATUS_WAITING4HASHCHECK]
+        exclude_states = [DLSTATUS_METADATA, DLSTATUS_CIRCUITS, DLSTATUS_EXIT_NODES,
+                          DLSTATUS_HASHCHECKING, DLSTATUS_WAITING4HASHCHECK]
         if len(self.selected_items) == 1 and self.selected_items[0].get_raw_download_status() not in exclude_states:
             menu.addAction(export_download_action)
-            menu.addSeparator()
-        menu_anon_level = menu.addMenu("Change anonymity")
+        menu.addAction(explore_files_action)
+        if len(self.selected_items) == 1:
+            menu.addAction(move_files_action)
+        menu.addSeparator()
+
+        menu_anon_level = menu.addMenu("Change Anonymity ")
         menu_anon_level.addAction(no_anon_action)
         menu_anon_level.addAction(one_hop_anon_action)
         menu_anon_level.addAction(two_hop_anon_action)
         menu_anon_level.addAction(three_hop_anon_action)
-        menu.addAction(explore_files_action)
 
         menu.exec_(self.window().downloads_list.mapToGlobal(pos))
