@@ -1,19 +1,36 @@
+from __future__ import absolute_import
+
 import logging
-from urllib import unquote_plus, url2pathname
+import os
+from binascii import hexlify, unhexlify
+
+from libtorrent import bencode, create_torrent
+
+from pony.orm import db_session
+
+from six import unichr  # pylint: disable=redefined-builtin
+from six.moves.urllib.parse import unquote_plus
+from six.moves.urllib.request import url2pathname
 
 from twisted.web import http, resource
 from twisted.web.server import NOT_DONE_YET
 
-from Tribler.Core.DownloadConfig import DownloadStartupConfig
-from Tribler.Core.Modules.restapi.util import return_handled_exception
-from Tribler.Core.Utilities.utilities import unichar_string
-from Tribler.Core.simpledefs import DOWNLOAD, UPLOAD, dlstatus_strings, DLMODE_VOD
 import Tribler.Core.Utilities.json_util as json
+from Tribler.Core.DownloadConfig import DownloadStartupConfig
+from Tribler.Core.Modules.MetadataStore.serialization import CHANNEL_TORRENT, ChannelMetadataPayload
+from Tribler.Core.Modules.MetadataStore.store import UNKNOWN_CHANNEL, UPDATED_OUR_VERSION
+from Tribler.Core.Modules.restapi.util import return_handled_exception
+from Tribler.Core.Utilities.torrent_utils import get_info_from_handle
+from Tribler.Core.Utilities.unicode import ensure_unicode
+from Tribler.Core.Utilities.utilities import unichar_string
+from Tribler.Core.exceptions import InvalidSignatureException
+from Tribler.Core.simpledefs import DLMODE_VOD, DOWNLOAD, UPLOAD, dlstatus_strings
+from Tribler.util import cast_to_unicode_utf8
 
 
 def _safe_extended_peer_info(ext_peer_info):
     """
-    Given a string describing peer info, return a JSON.dumps() safe representation.
+    Given a string describing peer info, return a json.twisted_dumps() safe representation.
 
     :param ext_peer_info: the string to convert to a dumpable format
     :return: the safe string
@@ -22,7 +39,7 @@ def _safe_extended_peer_info(ext_peer_info):
     if not ext_peer_info:
         ext_peer_info = u''
     try:
-        json.dumps(ext_peer_info)
+        json.twisted_dumps(ext_peer_info)
         return ext_peer_info
     except UnicodeDecodeError:
         # We might have some special unicode characters in here
@@ -45,7 +62,7 @@ class DownloadBaseEndpoint(resource.Resource):
         Returns a 404 response code if your channel has not been created.
         """
         request.setResponseCode(http.NOT_FOUND)
-        return json.dumps({"error": message})
+        return json.twisted_dumps({"error": message})
 
     @staticmethod
     def create_dconfig_from_params(parameters):
@@ -77,11 +94,11 @@ class DownloadBaseEndpoint(resource.Resource):
             download_config.set_safe_seeding(True)
 
         if 'destination' in parameters and len(parameters['destination']) > 0:
-            dest_dir = unicode(parameters['destination'][0], 'utf-8')
+            dest_dir = cast_to_unicode_utf8(parameters['destination'][0])
             download_config.set_dest_dir(dest_dir)
 
-        if 'selected_files[]' in parameters:
-            selected_files_list = [unicode(f, 'utf-8') for f in parameters['selected_files[]']]
+        if 'selected_files' in parameters:
+            selected_files_list = [cast_to_unicode_utf8(f) for f in parameters['selected_files']]
             download_config.set_selected_files(selected_files_list)
 
         return download_config, None
@@ -176,7 +193,6 @@ class DownloadsEndpoint(DownloadBaseEndpoint):
                             "dtotal": 23,
                             "downrate": 0,
                             "uinterested": False,
-                            "wstate": "\x00",
                             "optimistic": False,
                             ...
                         }, ...],
@@ -190,53 +206,75 @@ class DownloadsEndpoint(DownloadBaseEndpoint):
                 }, ...]
         """
         get_peers = False
-        if 'get_peers' in request.args and len(request.args['get_peers']) > 0 \
-                and request.args['get_peers'][0] == "1":
+        if b'get_peers' in request.args and len(request.args[b'get_peers']) > 0 \
+                and request.args[b'get_peers'][0] == "1":
             get_peers = True
 
         get_pieces = False
-        if 'get_pieces' in request.args and len(request.args['get_pieces']) > 0 \
-                and request.args['get_pieces'][0] == "1":
+        if b'get_pieces' in request.args and len(request.args[b'get_pieces']) > 0 \
+                and request.args[b'get_pieces'][0] == "1":
             get_pieces = True
 
-        get_files = 'get_files' in request.args and request.args['get_files'] and request.args['get_files'][0] == "1"
+        get_files = b'get_files' in request.args and request.args[b'get_files'] and request.args[b'get_files'][0] == "1"
 
         downloads_json = []
         downloads = self.session.get_downloads()
         for download in downloads:
+            if download.hidden and not download.get_channel_download():
+                # We still want to send channel downloads since they are displayed in the GUI
+                continue
             state = download.get_state()
             tdef = download.get_def()
 
             # Create tracker information of the download
             tracker_info = []
-            for url, url_info in download.get_tracker_status().iteritems():
+            for url, url_info in download.get_tracker_status().items():
                 tracker_info.append({"url": url, "peers": url_info[0], "status": url_info[1]})
 
             num_seeds, num_peers = state.get_num_seeds_peers()
             num_connected_seeds, num_connected_peers = download.get_num_connected_seeds_peers()
 
-            download_json = {"name": tdef.get_name_utf8(), "progress": state.get_progress(),
-                             "infohash": tdef.get_infohash().encode('hex'),
-                             "speed_down": state.get_current_payload_speed(DOWNLOAD),
-                             "speed_up": state.get_current_payload_speed(UPLOAD),
-                             "status": dlstatus_strings[state.get_status()],
-                             "size": tdef.get_length(), "eta": state.get_eta(),
-                             "num_peers": num_peers, "num_seeds": num_seeds,
-                             "num_connected_peers": num_connected_peers, "num_connected_seeds": num_connected_seeds,
-                             "total_up": state.get_total_transferred(UPLOAD),
-                             "total_down": state.get_total_transferred(DOWNLOAD), "ratio": state.get_seeding_ratio(),
-                             "trackers": tracker_info, "hops": download.get_hops(),
-                             "anon_download": download.get_anon_mode(), "safe_seeding": download.get_safe_seeding(),
-                             # Maximum upload/download rates are set for entire sessions
-                             "max_upload_speed": self.session.config.get_libtorrent_max_upload_rate(),
-                             "max_download_speed": self.session.config.get_libtorrent_max_download_rate(),
-                             "destination": download.get_dest_dir(), "availability": state.get_availability(),
-                             "total_pieces": tdef.get_nr_pieces(), "vod_mode": download.get_mode() == DLMODE_VOD,
-                             "vod_prebuffering_progress": state.get_vod_prebuffering_progress(),
-                             "vod_prebuffering_progress_consec": state.get_vod_prebuffering_progress_consec(),
-                             "error": repr(state.get_error()) if state.get_error() else "",
-                             "time_added": download.get_time_added(),
-                             "credit_mining": download.get_credit_mining()}
+            if download.get_channel_download():
+                download_name = self.session.lm.mds.ChannelMetadata.get_channel_name(
+                    tdef.get_name_utf8(), tdef.get_infohash())
+            else:
+                download_name = self.session.lm.mds.TorrentMetadata.get_torrent_title(tdef.get_infohash()) or \
+                                tdef.get_name_utf8()
+
+            download_json = {
+                "name": download_name,
+                "progress": state.get_progress(),
+                "infohash": hexlify(tdef.get_infohash()),
+                "speed_down": state.get_current_payload_speed(DOWNLOAD),
+                "speed_up": state.get_current_payload_speed(UPLOAD),
+                "status": dlstatus_strings[state.get_status()],
+                "size": tdef.get_length(),
+                "eta": state.get_eta(),
+                "num_peers": num_peers,
+                "num_seeds": num_seeds,
+                "num_connected_peers": num_connected_peers,
+                "num_connected_seeds": num_connected_seeds,
+                "total_up": state.get_total_transferred(UPLOAD),
+                "total_down": state.get_total_transferred(DOWNLOAD),
+                "ratio": state.get_seeding_ratio(),
+                "trackers": tracker_info,
+                "hops": download.get_hops(),
+                "anon_download": download.get_anon_mode(),
+                "safe_seeding": download.get_safe_seeding(),
+                # Maximum upload/download rates are set for entire sessions
+                "max_upload_speed": self.session.config.get_libtorrent_max_upload_rate(),
+                "max_download_speed": self.session.config.get_libtorrent_max_download_rate(),
+                "destination": download.get_dest_dir(),
+                "availability": state.get_availability(),
+                "total_pieces": tdef.get_nr_pieces(),
+                "vod_mode": download.get_mode() == DLMODE_VOD,
+                "vod_prebuffering_progress": state.get_vod_prebuffering_progress(),
+                "vod_prebuffering_progress_consec": state.get_vod_prebuffering_progress_consec(),
+                "error": repr(state.get_error()) if state.get_error() else "",
+                "time_added": download.get_time_added(),
+                "credit_mining": download.get_credit_mining(),
+                "channel_download": download.get_channel_download()
+            }
 
             # Add peers information if requested
             if get_peers:
@@ -245,7 +283,7 @@ class DownloadsEndpoint(DownloadBaseEndpoint):
                     del peer_info['have']
                     if 'extended_version' in peer_info:
                         peer_info['extended_version'] = _safe_extended_peer_info(peer_info['extended_version'])
-                    peer_info['id'] = peer_info['id'].encode('hex')
+                    peer_info['id'] = hexlify(peer_info['id'])
 
                 download_json["peers"] = peer_list
 
@@ -258,7 +296,7 @@ class DownloadsEndpoint(DownloadBaseEndpoint):
                 download_json["files"] = self.get_files_info_json(download)
 
             downloads_json.append(download_json)
-        return json.dumps({"downloads": downloads_json})
+        return json.twisted_dumps({"downloads": downloads_json})
 
     def render_PUT(self, request):
         """
@@ -288,28 +326,48 @@ class DownloadsEndpoint(DownloadBaseEndpoint):
 
         if 'uri' not in parameters or len(parameters['uri']) == 0:
             request.setResponseCode(http.BAD_REQUEST)
-            return json.dumps({"error": "uri parameter missing"})
+            return json.twisted_dumps({"error": "uri parameter missing"})
 
         download_config, error = DownloadsEndpoint.create_dconfig_from_params(parameters)
         if error:
             request.setResponseCode(http.BAD_REQUEST)
-            return json.dumps({"error": error})
+            return json.twisted_dumps({"error": error})
 
         def download_added(download):
-            request.write(json.dumps({"started": True,
-                                      "infohash": download.get_def().get_infohash().encode('hex')}))
+            request.write(json.twisted_dumps({"started": True,
+                                      "infohash": hexlify(download.get_def().get_infohash())}))
             request.finish()
 
         def on_error(error):
             request.setResponseCode(http.INTERNAL_SERVER_ERROR)
-            request.write(json.dumps({"error": unichar_string(error.getErrorMessage())}))
+            request.write(json.twisted_dumps({"error": unichar_string(error.getErrorMessage())}))
             request.finish()
 
-        uri = parameters['uri'][0]
+        uri = ensure_unicode(parameters['uri'][0], 'utf-8')
         if uri.startswith("file:"):
-            download_uri = u"file:%s" % url2pathname(uri[5:]).decode('utf-8')
+            filename = url2pathname(uri[5:])
+            if uri.endswith(".mdblob") or uri.endswith(".mdblob.lz4"):
+                with db_session:
+                    try:
+                        results = self.session.lm.mds.process_mdblob_file(filename)
+                        if results:
+                            node, status = results[0]
+                            if (status == UNKNOWN_CHANNEL or
+                                    (status == UPDATED_OUR_VERSION and node.metadata_type == CHANNEL_TORRENT)):
+                                node.subscribed = True
+                                return json.twisted_dumps(
+                                    {"started": True, "infohash": hexlify(node.infohash)})
+                        return json.twisted_dumps({"error": "Could not import Tribler metadata file"})
+                    except IOError:
+                        request.setResponseCode(http.BAD_REQUEST)
+                        return json.twisted_dumps({"error": "Metadata file not found"})
+                    except InvalidSignatureException:
+                        request.setResponseCode(http.BAD_REQUEST)
+                        return json.twisted_dumps({"error": "Metadata has invalid signature"})
+            else:
+                download_uri = u"file:%s" % filename
         else:
-            download_uri = unquote_plus(unicode(uri, 'utf-8'))
+            download_uri = unquote_plus(uri)
         download_deferred = self.session.start_download_from_uri(download_uri, download_config)
         download_deferred.addCallback(download_added)
         download_deferred.addErrback(on_error)
@@ -324,9 +382,9 @@ class DownloadSpecificEndpoint(DownloadBaseEndpoint):
 
     def __init__(self, session, infohash):
         DownloadBaseEndpoint.__init__(self, session)
-        self.infohash = bytes(infohash.decode('hex'))
-        self.putChild("torrent", DownloadExportTorrentEndpoint(session, self.infohash))
-        self.putChild("files", DownloadFilesEndpoint(session, self.infohash))
+        self.infohash = bytes(unhexlify(infohash))
+        self.putChild(b"torrent", DownloadExportTorrentEndpoint(session, self.infohash))
+        self.putChild(b"files", DownloadFilesEndpoint(session, self.infohash))
 
     def render_DELETE(self, request):
         """
@@ -352,7 +410,7 @@ class DownloadSpecificEndpoint(DownloadBaseEndpoint):
 
         if 'remove_data' not in parameters or len(parameters['remove_data']) == 0:
             request.setResponseCode(http.BAD_REQUEST)
-            return json.dumps({"error": "remove_data parameter missing"})
+            return json.twisted_dumps({"error": "remove_data parameter missing"})
 
         download = self.session.get_download(self.infohash)
         if not download:
@@ -364,8 +422,8 @@ class DownloadSpecificEndpoint(DownloadBaseEndpoint):
             """
             Success callback
             """
-            request.write(json.dumps({"removed": True,
-                                      "infohash": download.get_def().get_infohash().encode('hex')}))
+            request.write(json.twisted_dumps({"removed": True,
+                                      "infohash": hexlify(download.get_def().get_infohash())}))
             request.finish()
 
         def _on_remove_failure(failure):
@@ -387,7 +445,7 @@ class DownloadSpecificEndpoint(DownloadBaseEndpoint):
 
     def render_PATCH(self, request):
         """
-        .. http:patch:: /download/(string: infohash)
+        .. http:patch:: /downloads/(string: infohash)
 
         A PATCH request to this endpoint will update a download in Tribler.
 
@@ -422,7 +480,7 @@ class DownloadSpecificEndpoint(DownloadBaseEndpoint):
 
         if len(parameters) > 1 and 'anon_hops' in parameters:
             request.setResponseCode(http.BAD_REQUEST)
-            return json.dumps({"error": "anon_hops must be the only parameter in this request"})
+            return json.twisted_dumps({"error": "anon_hops must be the only parameter in this request"})
         elif 'anon_hops' in parameters:
             anon_hops = int(parameters['anon_hops'][0])
             deferred = self.session.lm.update_download_hops(download, anon_hops)
@@ -431,8 +489,8 @@ class DownloadSpecificEndpoint(DownloadBaseEndpoint):
                 """
                 Success callback
                 """
-                request.write(json.dumps({"modified": True,
-                                          "infohash": download.get_def().get_infohash().encode('hex')}))
+                request.write(json.twisted_dumps({"modified": True,
+                                          "infohash": hexlify(download.get_def().get_infohash())}))
                 request.finish()
 
             def _on_download_readd_failure(failure):
@@ -452,14 +510,14 @@ class DownloadSpecificEndpoint(DownloadBaseEndpoint):
             # As such, we can return immediately.
             return NOT_DONE_YET
 
-        if 'selected_files[]' in parameters:
+        if 'selected_files' in parameters:
             selected_files_list = []
-            for ind in parameters['selected_files[]']:
+            for ind in parameters['selected_files']:
                 try:
                     selected_files_list.append(download.tdef.get_files()[int(ind)])
                 except IndexError:  # File could not be found
                     request.setResponseCode(http.BAD_REQUEST)
-                    return json.dumps({"error": "index %s out of range" % ind})
+                    return json.twisted_dumps({"error": "index %s out of range" % ind})
             download.set_selected_files(selected_files_list)
 
         if 'state' in parameters and len(parameters['state']) > 0:
@@ -470,12 +528,18 @@ class DownloadSpecificEndpoint(DownloadBaseEndpoint):
                 download.stop()
             elif state == "recheck":
                 download.force_recheck()
+            elif state == "move_storage":
+                dest_dir = cast_to_unicode_utf8(parameters['dest_dir'][0])
+                if not os.path.exists(dest_dir):
+                    return json.twisted_dumps({"error": "Target directory (%s) does not exist" % dest_dir})
+                download.move_storage(dest_dir)
+                download.checkpoint()
             else:
                 request.setResponseCode(http.BAD_REQUEST)
-                return json.dumps({"error": "unknown state parameter"})
+                return json.twisted_dumps({"error": "unknown state parameter"})
 
-        return json.dumps({"modified": True,
-                           "infohash": download.get_def().get_infohash().encode('hex')})
+        return json.twisted_dumps({"modified": True,
+                                   "infohash": hexlify(download.get_def().get_infohash())})
 
 
 class DownloadExportTorrentEndpoint(DownloadBaseEndpoint):
@@ -503,13 +567,21 @@ class DownloadExportTorrentEndpoint(DownloadBaseEndpoint):
 
             The contents of the .torrent file.
         """
-        torrent = self.session.get_collected_torrent(self.infohash)
-        if not torrent:
-            return DownloadExportTorrentEndpoint.return_404(request)
+        download = self.session.get_download(self.infohash)
+        if not download:
+            return DownloadSpecificEndpoint.return_404(request)
+
+        if not download.handle or not download.handle.is_valid() or not download.handle.has_metadata():
+            return DownloadSpecificEndpoint.return_404(request)
+
+        torrent_info = get_info_from_handle(download.handle)
+        t = create_torrent(torrent_info)
+        torrent = t.generate()
+        bencoded_torrent = bencode(torrent)
 
         request.setHeader(b'content-type', 'application/x-bittorrent')
-        request.setHeader(b'Content-Disposition', 'attachment; filename=%s.torrent' % self.infohash.encode('hex'))
-        return torrent
+        request.setHeader(b'Content-Disposition', 'attachment; filename=%s.torrent' % hexlify(self.infohash))
+        return bencoded_torrent
 
 
 class DownloadFilesEndpoint(DownloadBaseEndpoint):
@@ -551,4 +623,4 @@ class DownloadFilesEndpoint(DownloadBaseEndpoint):
         if not download:
             return DownloadExportTorrentEndpoint.return_404(request)
 
-        return json.dumps({"files": self.get_files_info_json(download)})
+        return json.twisted_dumps({"files": self.get_files_info_json(download)})
