@@ -9,14 +9,15 @@ Community instance.
 """
 from __future__ import absolute_import
 
-from binascii import hexlify
 import sys
+from binascii import hexlify
 from random import choice, random
 from socket import error, gethostbyname
 from time import time
 from traceback import format_exception
 
-from .lazy_community import lazy_wrapper, lazy_wrapper_unsigned, EZPackOverlay
+from .lazy_community import EZPackOverlay, lazy_wrapper, lazy_wrapper_unsigned
+from .messaging.anonymization.endpoint import TunnelEndpoint
 from .messaging.payload import (IntroductionRequestPayload, IntroductionResponsePayload, PuncturePayload,
                                 PunctureRequestPayload)
 from .messaging.payload_headers import BinMemberAuthenticationPayload, GlobalTimeDistributionPayload
@@ -68,13 +69,20 @@ class Community(EZPackOverlay):
     version = b'\x02'
     master_peer = ""
 
-    def __init__(self, my_peer, endpoint, network, max_peers=DEFAULT_MAX_PEERS):
+    def __init__(self, my_peer, endpoint, network, max_peers=DEFAULT_MAX_PEERS, anonymize=False):
         super(Community, self).__init__(self.master_peer, my_peer, endpoint, network)
 
-        self._prefix = b'\x00' + self.version + self.master_peer.key.key_to_hash()
+        self._prefix = b'\x00' + self.version + self.master_peer.mid
         self.logger.debug("Launching %s with prefix %s.", self.__class__.__name__, hexlify(self._prefix))
 
         self.max_peers = max_peers
+        self.anonymize = anonymize
+
+        if anonymize:
+            if isinstance(self.endpoint, TunnelEndpoint):
+                self.endpoint.set_anonymity(self._prefix, True)
+            else:
+                self.logger.warning('Cannot anonymize community traffic without TunnelEndpoint')
 
         self.network.register_service_provider(self.master_peer.mid, self)
         self.network.blacklist_mids.append(my_peer.mid)
@@ -130,6 +138,24 @@ class Community(EZPackOverlay):
     def get_prefix(self):
         return self._prefix
 
+    def add_message_handler(self, msg_num, callback):
+        """
+        Add a handler for a message identifier. Any messages coming in with this identifier will be delivered to
+        the specified callback function.
+
+        :param msg_num: the message id to listen for
+        :type msg_num: int
+        :param callback: the callback function for this message id
+        :type callback: function
+        :returns: None
+        """
+        if msg_num < 0 or msg_num > 255:
+            raise RuntimeError("Attempted to add a handler for message number %d, which is not a byte!" % msg_num)
+        if chr(msg_num) in self.decode_map:
+            raise RuntimeError("Attempted to add a handler for message number %d, already mapped to %s!" %
+                               (msg_num, self.decode_map[chr(msg_num)]))
+        self.decode_map[chr(msg_num)] = callback
+
     def on_deprecated_message(self, source_address, data):
         self.logger.warning("Received deprecated message: %s from (%s, %d)",
                             self.deprecated_message_names[data[22]], *source_address)
@@ -166,8 +192,8 @@ class Community(EZPackOverlay):
     def create_introduction_response(self, lan_socket_address, socket_address, identifier,
                                      introduction=None, extra_bytes=b''):
         global_time = self.claim_global_time()
-        introduction_lan = ("0.0.0.0",0)
-        introduction_wan = ("0.0.0.0",0)
+        introduction_lan = ("0.0.0.0", 0)
+        introduction_wan = ("0.0.0.0", 0)
         introduced = False
         other = self.network.get_verified_by_address(socket_address)
         if not introduction:
@@ -193,7 +219,7 @@ class Community(EZPackOverlay):
 
         if introduced:
             packet = self.create_puncture_request(lan_socket_address, socket_address, identifier)
-            self.endpoint.send(introduction_wan if introduction_lan == ("0.0.0.0",0) else introduction_lan, packet)
+            self.endpoint.send(introduction_wan if introduction_lan == ("0.0.0.0", 0) else introduction_lan, packet)
 
         return self._ez_pack(self._prefix, 245, [auth, dist, payload])
 
@@ -258,11 +284,14 @@ class Community(EZPackOverlay):
         self.network.add_verified_peer(peer)
         self.network.discover_services(peer, [self.master_peer.mid, ])
 
-        if (payload.wan_introduction_address != ("0.0.0.0", 0) and
-                payload.wan_introduction_address[0] != self.my_estimated_wan[0]):
+        if (payload.wan_introduction_address != ("0.0.0.0", 0)
+                and payload.wan_introduction_address[0] != self.my_estimated_wan[0]):
+            if (payload.lan_introduction_address != ("0.0.0.0", 0)
+                    and self.my_estimated_wan[0] == self.my_estimated_lan[0]):
+                self.network.discover_address(peer, payload.lan_introduction_address, self.master_peer.mid)
             self.network.discover_address(peer, payload.wan_introduction_address, self.master_peer.mid)
-        elif (payload.lan_introduction_address != ("0.0.0.0", 0) and
-              payload.wan_introduction_address[0] == self.my_estimated_wan[0]):
+        elif (payload.lan_introduction_address != ("0.0.0.0", 0)
+              and payload.wan_introduction_address[0] == self.my_estimated_wan[0]):
             self.network.discover_address(peer, payload.lan_introduction_address, self.master_peer.mid)
         elif payload.wan_introduction_address != ("0.0.0.0", 0):
             self.network.discover_address(peer, payload.wan_introduction_address, self.master_peer.mid)
@@ -273,8 +302,7 @@ class Community(EZPackOverlay):
 
     @lazy_wrapper(GlobalTimeDistributionPayload, PuncturePayload)
     def on_puncture(self, peer, dist, payload):
-        self.network.add_verified_peer(peer)
-        self.network.discover_services(peer, [self.master_peer.mid, ])
+        pass
 
     @lazy_wrapper_unsigned(GlobalTimeDistributionPayload, PunctureRequestPayload)
     def on_puncture_request(self, source_address, dist, payload):
@@ -297,8 +325,8 @@ class Community(EZPackOverlay):
             try:
                 self.decode_map[msg_id](source_address, data)
             except:
-                self.logger.error("Exception occurred while handling packet!\n" +
-                                  ''.join(format_exception(*sys.exc_info())))
+                self.logger.error("Exception occurred while handling packet!\n"
+                                  + ''.join(format_exception(*sys.exc_info())))
         elif warn_unknown:
             self.logger.warning("Received unknown message: %d from (%s, %d)", ord(msg_id), *source_address)
 
@@ -306,18 +334,14 @@ class Community(EZPackOverlay):
         packet = self.create_introduction_request(address)
         self.endpoint.send(address, packet)
 
-    def send_introduction_request(self, peer, service_id=None):
+    def send_introduction_request(self, peer):
         """
         Send an introduction request to a specific peer.
         """
         packet = self.create_introduction_request(peer.address)
-
-        if service_id:
-            packet = packet[:2] + service_id + packet[22:]
-
         self.endpoint.send(peer.address, packet)
 
-    def get_new_introduction(self, from_peer=None, service_id=None):
+    def get_new_introduction(self, from_peer=None):
         """
         Get a new introduction, or bootstrap if there are no available peers.
         """
@@ -334,10 +358,6 @@ class Community(EZPackOverlay):
                 return
 
         packet = self.create_introduction_request(from_peer)
-
-        if service_id:
-            packet = packet[:2] + service_id + packet[22:]
-
         self.endpoint.send(from_peer, packet)
 
     def get_peer_for_introduction(self, exclude=None):
@@ -347,5 +367,8 @@ class Community(EZPackOverlay):
         available = [p for p in self.get_peers() if p != exclude]
         return choice(available) if available else None
 
+    def get_walkable_addresses(self):
+        return self.network.get_walkable_addresses(self.master_peer.mid)
+
     def get_peers(self):
-        return self.network.get_peers_for_service(self.master_peer.key.key_to_hash())
+        return self.network.get_peers_for_service(self.master_peer.mid)
