@@ -7,41 +7,41 @@ import signal
 import sys
 import time
 import traceback
-import urlparse
-from urllib import pathname2url, unquote
+from base64 import b64encode
+from binascii import hexlify
+
 
 from PyQt5 import uic
-from PyQt5.QtCore import QCoreApplication, QObject, QPoint, QSettings, QStringListModel, QTimer, QUrl, Qt, \
-    pyqtSignal, pyqtSlot
-from PyQt5.QtCore import QDir
-from PyQt5.QtGui import QDesktopServices, QIcon
-from PyQt5.QtGui import QKeySequence
-from PyQt5.QtGui import QPixmap
-from PyQt5.QtWidgets import QAction, QApplication, QCompleter, QFileDialog, QLineEdit, QListWidget, QMainWindow, \
-    QStyledItemDelegate, QSystemTrayIcon, QTreeWidget
-from PyQt5.QtWidgets import QShortcut
+from PyQt5.QtCore import (
+    QCoreApplication, QDir, QObject, QPoint, QSettings, QStringListModel, QTimer, QUrl, Qt, pyqtSignal, pyqtSlot)
+from PyQt5.QtGui import QDesktopServices, QIcon, QKeySequence, QPixmap
+from PyQt5.QtWidgets import (
+    QAction, QApplication, QCompleter, QFileDialog, QLineEdit, QListWidget, QMainWindow, QShortcut, QStyledItemDelegate,
+    QSystemTrayIcon, QTreeWidget)
 
 import six
+from six.moves.urllib.parse import unquote, urlparse
 
 from Tribler.Core.Modules.process_checker import ProcessChecker
 
 from TriblerGUI.core_manager import CoreManager
 from TriblerGUI.debug_window import DebugWindow
-from TriblerGUI.defs import BUTTON_TYPE_CONFIRM, BUTTON_TYPE_NORMAL, DEFAULT_API_PORT, PAGE_CHANNEL_DETAILS, \
-    PAGE_DISCOVERED, PAGE_DISCOVERING, PAGE_DOWNLOADS, PAGE_EDIT_CHANNEL, PAGE_HOME, PAGE_LOADING, \
-    PAGE_PLAYLIST_DETAILS, PAGE_SEARCH_RESULTS, PAGE_SETTINGS, PAGE_SUBSCRIBED_CHANNELS, PAGE_TRUST, \
-    PAGE_VIDEO_PLAYER, SHUTDOWN_WAITING_PERIOD
+from TriblerGUI.defs import (
+    BUTTON_TYPE_CONFIRM, BUTTON_TYPE_NORMAL, CONTEXT_MENU_WIDTH, DEFAULT_API_PORT, PAGE_CHANNEL_DETAILS,
+    PAGE_DISCOVERED, PAGE_DISCOVERING, PAGE_DOWNLOADS, PAGE_EDIT_CHANNEL, PAGE_HOME, PAGE_LOADING,
+    PAGE_SEARCH_RESULTS, PAGE_SETTINGS, PAGE_SUBSCRIBED_CHANNELS, PAGE_TRUST, PAGE_TRUST_GRAPH_PAGE,
+    PAGE_VIDEO_PLAYER, SHUTDOWN_WAITING_PERIOD)
 from TriblerGUI.dialogs.confirmationdialog import ConfirmationDialog
+from TriblerGUI.dialogs.createtorrentdialog import CreateTorrentDialog
 from TriblerGUI.dialogs.feedbackdialog import FeedbackDialog
 from TriblerGUI.dialogs.startdownloaddialog import StartDownloadDialog
 from TriblerGUI.tribler_action_menu import TriblerActionMenu
 from TriblerGUI.tribler_request_manager import TriblerRequestManager, dispatcher, request_queue
-from TriblerGUI.utilities import get_gui_setting, get_image_path, get_ui_file_path, is_dir_writable, quote_plus_unicode
+from TriblerGUI.utilities import get_gui_setting, get_image_path, get_ui_file_path, is_dir_writable
+from TriblerGUI.widgets.triblertablecontrollers import sanitize_for_fts
 
 # Pre-load form UI classes
-fc_channel_torrent_list_item, _ = uic.loadUiType(get_ui_file_path('channel_torrent_list_item.ui'))
-fc_channel_list_item, _ = uic.loadUiType(get_ui_file_path('channel_list_item.ui'))
-fc_playlist_list_item, _ = uic.loadUiType(get_ui_file_path('playlist_list_item.ui'))
+
 fc_home_recommended_item, _ = uic.loadUiType(get_ui_file_path('home_recommended_item.ui'))
 fc_loading_list_item, _ = uic.loadUiType(get_ui_file_path('loading_list_item.ui'))
 
@@ -89,8 +89,7 @@ class TriblerWindow(QMainWindow):
         if self.debug_window:
             self.debug_window.setHidden(True)
 
-        dialog = FeedbackDialog(self, exception_text, self.core_manager.events_manager.tribler_version,
-                                self.start_time)
+        dialog = FeedbackDialog(self, exception_text, self.tribler_version, self.start_time)
         dialog.show()
 
     def __init__(self, core_args=None, core_env=None, api_port=None):
@@ -108,12 +107,15 @@ class TriblerWindow(QMainWindow):
         self.navigation_stack = []
         self.tribler_started = False
         self.tribler_settings = None
+        self.tribler_version = None
         self.debug_window = None
         self.core_manager = CoreManager(api_port)
         self.pending_requests = {}
         self.pending_uri_requests = []
         self.download_uri = None
         self.dialog = None
+        self.create_dialog = None
+        self.chosen_dir = None
         self.new_version_dialog = None
         self.start_download_dialog_active = False
         self.request_mgr = None
@@ -128,6 +130,7 @@ class TriblerWindow(QMainWindow):
         self.exception_handler_called = False
         self.token_refresh_timer = None
         self.shutdown_timer = None
+        self.add_torrent_url_dialog_active = False
 
         sys.excepthook = self.on_exception
 
@@ -135,19 +138,6 @@ class TriblerWindow(QMainWindow):
         TriblerRequestManager.window = self
         self.tribler_status_bar.hide()
 
-        # Load dynamic widgets
-        uic.loadUi(get_ui_file_path('torrent_channel_list_container.ui'), self.channel_page_container)
-        self.channel_torrents_list = self.channel_page_container.items_list
-        self.channel_torrents_detail_widget = self.channel_page_container.details_tab_widget
-        self.channel_torrents_detail_widget.initialize_details_widget()
-        self.channel_torrents_list.itemSelectionChanged.connect(self.channel_page.clicked_item)
-
-        uic.loadUi(get_ui_file_path('torrent_channel_list_container.ui'), self.search_page_container)
-        self.search_results_list = self.search_page_container.items_list
-        self.search_torrents_detail_widget = self.search_page_container.details_tab_widget
-        self.search_torrents_detail_widget.initialize_details_widget()
-        self.search_results_list.itemClicked.connect(self.on_channel_item_click)
-        self.search_results_list.itemSelectionChanged.connect(self.search_results_page.clicked_item)
         self.token_balance_widget.mouseReleaseEvent = self.on_token_balance_click
 
         def on_state_update(new_state):
@@ -160,6 +150,10 @@ class TriblerWindow(QMainWindow):
 
         self.debug_pane_shortcut = QShortcut(QKeySequence("Ctrl+d"), self)
         self.debug_pane_shortcut.activated.connect(self.clicked_menu_button_debug)
+        self.import_torrent_shortcut = QShortcut(QKeySequence("Ctrl+o"), self)
+        self.import_torrent_shortcut.activated.connect(self.on_add_torrent_browse_file)
+        self.add_torrent_url_shortcut = QShortcut(QKeySequence("Ctrl+i"), self)
+        self.add_torrent_url_shortcut.activated.connect(self.on_add_torrent_from_url)
 
         # Remove the focus rect on OS X
         for widget in self.findChildren(QLineEdit) + self.findChildren(QListWidget) + self.findChildren(QTreeWidget):
@@ -167,20 +161,23 @@ class TriblerWindow(QMainWindow):
 
         self.menu_buttons = [self.left_menu_button_home, self.left_menu_button_search, self.left_menu_button_my_channel,
                              self.left_menu_button_subscriptions, self.left_menu_button_video_player,
-                             self.left_menu_button_downloads, self.left_menu_button_discovered]
+                             self.left_menu_button_downloads, self.left_menu_button_discovered,
+                             self.left_menu_button_trust_graph]
 
         self.video_player_page.initialize_player()
-        self.search_results_page.initialize_search_results_page()
+        self.search_results_page.initialize_search_results_page(self.gui_settings)
         self.settings_page.initialize_settings_page()
         self.subscribed_channels_page.initialize()
-        self.edit_channel_page.initialize_edit_channel_page()
+        self.edit_channel_page.initialize_edit_channel_page(self.gui_settings)
         self.downloads_page.initialize_downloads_page()
         self.home_page.initialize_home_page()
         self.loading_page.initialize_loading_page()
         self.discovering_page.initialize_discovering_page()
-        self.discovered_page.initialize_discovered_page()
+        self.discovered_page.initialize_discovered_page(self.gui_settings)
+        self.channel_page.initialize_channel_page(self.gui_settings)
         self.trust_page.initialize_trust_page()
         self.token_mining_page.initialize_token_mining_page()
+        self.trust_graph_page.initialize_trust_graph()
 
         self.stackedWidget.setCurrentIndex(PAGE_LOADING)
 
@@ -249,18 +246,15 @@ class TriblerWindow(QMainWindow):
         # Start Tribler
         self.core_manager.start(core_args=core_args, core_env=core_env)
 
-        self.core_manager.events_manager.received_search_result_channel.connect(
-            self.search_results_page.received_search_result_channel)
-        self.core_manager.events_manager.received_search_result_torrent.connect(
-            self.search_results_page.received_search_result_torrent)
         self.core_manager.events_manager.torrent_finished.connect(self.on_torrent_finished)
         self.core_manager.events_manager.new_version_available.connect(self.on_new_version_available)
         self.core_manager.events_manager.tribler_started.connect(self.on_tribler_started)
-        self.core_manager.events_manager.events_started.connect(self.on_events_started)
         self.core_manager.events_manager.low_storage_signal.connect(self.on_low_storage)
         self.core_manager.events_manager.credit_mining_signal.connect(self.on_credit_mining_error)
         self.core_manager.events_manager.tribler_shutdown_signal.connect(self.on_tribler_shutdown_state_update)
 
+        self.core_manager.events_manager.received_search_result.connect(self.search_results_page.received_search_result)
+
         # Install signal handler for ctrl+c events
         def sigint_handler(*_):
             self.close_tribler()
@@ -315,7 +309,8 @@ class TriblerWindow(QMainWindow):
         close_dialog.show()
 
     def on_torrent_finished(self, torrent_info):
-        self.tray_show_message("Download finished", "Download of %s has finished." % torrent_info["name"])
+        if "hidden" not in torrent_info or not torrent_info["hidden"]:
+            self.tray_show_message("Download finished", "Download of %s has finished." % torrent_info["name"])
 
     def show_loading_screen(self):
         self.top_menu_button.setHidden(True)
@@ -351,8 +346,12 @@ class TriblerWindow(QMainWindow):
             except RuntimeError as e:
                 logging.error("Failed to set tray message: %s", str(e))
 
-    def on_tribler_started(self):
+    def on_tribler_started(self, version):
+        if self.tribler_started:
+            logging.warning("Received duplicate Tribler Core started event")
+            return
         self.tribler_started = True
+        self.tribler_version = version
 
         self.top_menu_button.setHidden(False)
         self.left_menu.setHidden(False)
@@ -375,6 +374,10 @@ class TriblerWindow(QMainWindow):
             self.clicked_menu_button_home()
 
         self.setAcceptDrops(True)
+        self.setWindowTitle("Tribler %s" % self.tribler_version)
+
+        # Load channel if it exists
+        self.edit_channel_page.load_my_channel_overview()
 
     def on_events_started(self, json_dict):
         self.setWindowTitle("Tribler %s" % json_dict["version"])
@@ -394,11 +397,19 @@ class TriblerWindow(QMainWindow):
             return
 
         uri = self.pending_uri_requests.pop()
+
+        # TODO: create a proper confirmation dialog to show results of adding .mdblob files
+        # the case for .mdblob files is handled without torrentinfo endpoint
+        if uri.startswith('file') and (uri.endswith('.mdblob') or uri.endswith('.mdblob.lz4')):
+            request_mgr = TriblerRequestManager()
+            request_mgr.perform_request("downloads", lambda _: None, method='PUT', data={"uri": uri})
+            return
+
         if uri.startswith('file') or uri.startswith('magnet'):
             self.start_download_from_uri(uri)
 
     def perform_start_download_request(self, uri, anon_download, safe_seeding, destination, selected_files,
-                                       total_files=0, callback=None):
+                                       total_files=0, add_to_channel=False, callback=None):
         # Check if destination directory is writable
         is_writable, error = is_dir_writable(destination)
         if not is_writable:
@@ -408,18 +419,26 @@ class TriblerWindow(QMainWindow):
             ConfirmationDialog.show_message(self.window(), "Download error <i>%s</i>" % uri, gui_error_message, "OK")
             return
 
-        selected_files_uri = ""
+        if add_to_channel and not self.edit_channel_page.channel_overview:
+            error_message = "In the download options, you checked to add the torrent to your channel " \
+                            "but you do not have one yet. No worries, you can easily create your own channel by " \
+                            "navigating to My Channel section on the left sidebar."
+            ConfirmationDialog.show_error(self.window(), "Download Error!", error_message)
+            return
+
+        selected_files_list = []
         if len(selected_files) != total_files:  # Not all files included
-            selected_files_uri = u'&' + u''.join(u"selected_files[]=%s&" %
-                                                 quote_plus_unicode(filename) for filename in selected_files)[:-1]
+            selected_files_list = [filename for filename in selected_files]
 
         anon_hops = int(self.tribler_settings['download_defaults']['number_hops']) if anon_download else 0
         safe_seeding = 1 if safe_seeding else 0
-        post_data = "uri=%s&anon_hops=%d&safe_seeding=%d&destination=%s%s" % (quote_plus_unicode(uri), anon_hops,
-                                                                              safe_seeding, destination,
-                                                                              selected_files_uri)
-        post_data = post_data.encode('utf-8')  # We need to send bytes in the request, not unicode
-
+        post_data = {
+            "uri": uri,
+            "anon_hops": anon_hops,
+            "safe_seeding": safe_seeding,
+            "destination": destination,
+            "selected_files": selected_files_list
+        }
         request_mgr = TriblerRequestManager()
         request_mgr.perform_request("downloads", callback if callback else self.on_download_added,
                                     method='PUT', data=post_data)
@@ -429,7 +448,7 @@ class TriblerWindow(QMainWindow):
         recent_locations = current_settings.split(",") if len(current_settings) > 0 else []
         if isinstance(destination, six.text_type):
             destination = destination.encode('utf-8')
-        encoded_destination = destination.encode('hex')
+        encoded_destination = hexlify(destination)
         if encoded_destination in recent_locations:
             recent_locations.remove(encoded_destination)
         recent_locations.insert(0, encoded_destination)
@@ -439,6 +458,34 @@ class TriblerWindow(QMainWindow):
 
         self.gui_settings.setValue("recent_download_locations", ','.join(recent_locations))
 
+        if add_to_channel:
+            self.add_torrent_to_channel(uri)
+
+    def add_torrent_to_channel(self, uri, callback=None):
+        post_data = dict()
+        if uri.startswith("file:"):
+            with open(uri[5:], "rb") as torrent_file:
+                post_data['torrent'] = b64encode(torrent_file.read())
+        elif uri.startswith("magnet:"):
+            post_data['uri'] = uri
+
+        if post_data:
+            request_mgr = TriblerRequestManager()
+            request_mgr.perform_request("mychannel/torrents", callback if callback else lambda _: None,
+                                        method='PUT', data=post_data)
+
+    def add_dir_to_channel(self, dirname, recursive=False, callback=None):
+        post_data = {
+            "torrents_dir": dirname,
+            "recursive": int(recursive)
+        }
+        request_mgr = TriblerRequestManager()
+        request_mgr.perform_request("mychannel/torrents", callback if callback else lambda _: None,
+                                    method='PUT', data=post_data)
+
+    def on_dir_added_to_channel(self):
+        self.tray_show_message("Tribler update", "%s added to your channel" % self.chosen_dir)
+
     def on_new_version_available(self, version):
         if version == str(self.gui_settings.value('last_reported_version')):
             return
@@ -464,7 +511,7 @@ class TriblerWindow(QMainWindow):
     def on_search_text_change(self, text):
         self.search_suggestion_mgr = TriblerRequestManager()
         self.search_suggestion_mgr.perform_request(
-            "search/completions?q=%s" % text, self.on_received_search_completions)
+            "search/completions", self.on_received_search_completions, url_params={'q': sanitize_for_fts(text)})
 
     def on_received_search_completions(self, completions):
         if completions is None:
@@ -483,14 +530,18 @@ class TriblerWindow(QMainWindow):
         if 'error' in settings:
             raise RuntimeError(TriblerRequestManager.get_message_from_error(settings))
 
+        # If there is any pending dialog (likely download dialog or error dialog of setting not available),
+        # close the dialog
+        if self.dialog:
+            self.dialog.close_dialog()
+            self.dialog = None
+
         self.tribler_settings = settings['settings']
 
         # Set the video server port
         self.video_player_page.video_player_port = settings["ports"]["video_server~port"]
 
         # Disable various components based on the settings
-        if not self.tribler_settings['search_community']['enabled']:
-            self.window().top_search_bar.setHidden(True)
         if not self.tribler_settings['video_server']['enabled']:
             self.left_menu_button_video_player.setHidden(True)
         self.downloads_creditmining_button.setHidden(not self.tribler_settings["credit_mining"]["enabled"])
@@ -521,8 +572,6 @@ class TriblerWindow(QMainWindow):
         self.has_search_results = True
         self.clicked_menu_button_search()
         self.search_results_page.perform_search(current_search_query)
-        self.search_request_mgr = TriblerRequestManager()
-        self.search_request_mgr.perform_request("search?q=%s" % current_search_query, None)
         self.last_search_query = current_search_query
         self.last_search_time = current_ts
 
@@ -554,19 +603,16 @@ class TriblerWindow(QMainWindow):
         self.trust_page.received_trustchain_statistics(statistics)
 
         statistics = statistics["statistics"]
-        if 'latest_block' in statistics:
-            balance = (statistics["latest_block"]["transaction"]["total_up"] -
-                       statistics["latest_block"]["transaction"]["total_down"])
-            self.set_token_balance(balance)
-        else:
-            self.token_balance_label.setText("0 MB")
+        balance = (statistics.get("total_up", 0) -
+                   statistics.get("total_down", 0))
+        self.set_token_balance(balance)
 
         # If trust page is currently visible, then load the graph as well
         if self.stackedWidget.currentIndex() == PAGE_TRUST:
             self.trust_page.load_blocks()
 
     def set_token_balance(self, balance):
-        if abs(balance) > 1024 ** 4:    # Balance is over a TB
+        if abs(balance) > 1024 ** 4:  # Balance is over a TB
             balance /= 1024.0 ** 4
             self.token_balance_label.setText("%.1f TB" % balance)
         elif abs(balance) > 1024 ** 3:  # Balance is over a GB
@@ -590,19 +636,41 @@ class TriblerWindow(QMainWindow):
         browse_files_action = QAction('Import torrent from file', self)
         browse_directory_action = QAction('Import torrent(s) from directory', self)
         add_url_action = QAction('Import torrent from magnet/URL', self)
+        add_mdblob_action = QAction('Import Tribler metadata from file', self)
+        create_torrent_action = QAction('Create torrent from file(s)', self)
 
         browse_files_action.triggered.connect(self.on_add_torrent_browse_file)
         browse_directory_action.triggered.connect(self.on_add_torrent_browse_dir)
         add_url_action.triggered.connect(self.on_add_torrent_from_url)
+        add_mdblob_action.triggered.connect(self.on_add_mdblob_browse_file)
+        create_torrent_action.triggered.connect(self.on_create_torrent)
 
         menu.addAction(browse_files_action)
         menu.addAction(browse_directory_action)
         menu.addAction(add_url_action)
+        menu.addAction(add_mdblob_action)
+        menu.addSeparator()
+        menu.addAction(create_torrent_action)
 
         return menu
 
-    def on_add_torrent_button_click(self, pos):
-        self.create_add_torrent_menu().exec_(self.mapToGlobal(self.add_torrent_button.pos()))
+    def on_create_torrent(self):
+        if self.create_dialog:
+            self.create_dialog.close_dialog()
+
+        self.create_dialog = CreateTorrentDialog(self)
+        self.create_dialog.create_torrent_notification.connect(self.on_create_torrent_updates)
+        self.create_dialog.show()
+
+    def on_create_torrent_updates(self, update_dict):
+        self.tray_show_message("Torrent updates", update_dict['msg'])
+
+    def on_add_torrent_button_click(self, _pos):
+        plus_btn_pos = self.add_torrent_button.pos()
+        plus_btn_geometry = self.add_torrent_button.geometry()
+        plus_btn_pos.setX(plus_btn_pos.x() - CONTEXT_MENU_WIDTH)
+        plus_btn_pos.setY(plus_btn_pos.y() + plus_btn_geometry.height())
+        self.create_add_torrent_menu().exec_(self.mapToGlobal(plus_btn_pos))
 
     def on_add_torrent_browse_file(self):
         filenames = QFileDialog.getOpenFileNames(self,
@@ -610,18 +678,33 @@ class TriblerWindow(QMainWindow):
                                                  QDir.homePath(),
                                                  "Torrent files (*.torrent)")
         if len(filenames[0]) > 0:
-            [self.pending_uri_requests.append(u"file:%s" % filename) for filename in filenames[0]]
+            for filename in filenames[0]:
+                self.pending_uri_requests.append(u"file:%s" % filename)
+            self.process_uri_request()
+
+    def on_add_mdblob_browse_file(self):
+        filenames = QFileDialog.getOpenFileNames(self,
+                                                 "Please select the .mdblob file",
+                                                 QDir.homePath(),
+                                                 "Tribler metadata files (*.mdblob.lz4)")
+        if len(filenames[0]) > 0:
+            for filename in filenames[0]:
+                self.pending_uri_requests.append(u"file:%s" % filename)
             self.process_uri_request()
 
     def start_download_from_uri(self, uri):
         self.download_uri = uri
 
         if get_gui_setting(self.gui_settings, "ask_download_settings", True, is_bool=True):
+            # FIXME: instead of using this workaround, make sure the settings are _available_ by this moment
             # If tribler settings is not available, fetch the settings and inform the user to try again.
             if not self.tribler_settings:
                 self.fetch_settings()
-                ConfirmationDialog.show_error(self, "Download Error", "Tribler settings is not available yet. "
-                                                                      "Fetching it now. Please try again later.")
+                self.dialog = ConfirmationDialog.show_error(self, "Download Error", "Tribler settings is not available\
+                                                                   yet. Fetching it now. Please try again later.")
+                # By re-adding the download uri to the pending list, the request is re-processed
+                # when the settings is received
+                self.pending_uri_requests.append(uri)
                 return
             # Clear any previous dialog if exists
             if self.dialog:
@@ -633,6 +716,7 @@ class TriblerWindow(QMainWindow):
             self.dialog.show()
             self.start_download_dialog_active = True
         else:
+            # FIXME: instead of using this workaround, make sure the settings are _available_ by this moment
             # In the unlikely scenario that tribler settings are not available yet, try to fetch settings again and
             # add the download uri back to self.pending_uri_requests to process again.
             if not self.tribler_settings:
@@ -657,7 +741,8 @@ class TriblerWindow(QMainWindow):
                     self.dialog.dialog_widget.safe_seed_checkbox.isChecked(),
                     self.dialog.dialog_widget.destination_input.currentText(),
                     self.dialog.get_selected_files(),
-                    self.dialog.dialog_widget.files_list_view.topLevelItemCount())
+                    self.dialog.dialog_widget.files_list_view.topLevelItemCount(),
+                    add_to_channel=self.dialog.dialog_widget.add_to_channel_checkbox.isChecked())
             else:
                 ConfirmationDialog.show_error(self, "Tribler UI Error", "Something went wrong. Please try again.")
                 logging.exception("Error while trying to download. Either dialog or dialog.dialog_widget is None")
@@ -675,25 +760,28 @@ class TriblerWindow(QMainWindow):
                                                       "Please select the directory containing the .torrent files",
                                                       QDir.homePath(),
                                                       QFileDialog.ShowDirsOnly)
-
+        self.chosen_dir = chosen_dir
         if len(chosen_dir) != 0:
             self.selected_torrent_files = [torrent_file for torrent_file in glob.glob(chosen_dir + "/*.torrent")]
             self.dialog = ConfirmationDialog(self, "Add torrents from directory",
-                                             "Are you sure you want to add %d torrents to Tribler?" %
-                                             len(self.selected_torrent_files),
-                                             [('ADD', BUTTON_TYPE_NORMAL), ('CANCEL', BUTTON_TYPE_CONFIRM)])
+                                             "Add %s torrent files from the following directory "
+                                             "to your Tribler channel:\n\n%s" %
+                                             (len(self.selected_torrent_files), chosen_dir),
+                                             [('ADD', BUTTON_TYPE_NORMAL), ('CANCEL', BUTTON_TYPE_CONFIRM)],
+                                             checkbox_text="Add torrents to My Channel",)
             self.dialog.button_clicked.connect(self.on_confirm_add_directory_dialog)
             self.dialog.show()
 
     def on_confirm_add_directory_dialog(self, action):
         if action == 0:
+            if self.dialog.checkbox.isChecked():
+                self.add_dir_to_channel(self.chosen_dir, callback=self.on_dir_added_to_channel)
             for torrent_file in self.selected_torrent_files:
-                escaped_uri = u"file:%s" % pathname2url(torrent_file.encode('utf-8'))
-                self.perform_start_download_request(escaped_uri,
+                self.perform_start_download_request(u"file:%s" % torrent_file,
                                                     self.window().tribler_settings['download_defaults'][
-                                                         'anonymity_enabled'],
+                                                        'anonymity_enabled'],
                                                     self.window().tribler_settings['download_defaults'][
-                                                         'safeseeding_enabled'],
+                                                        'safeseeding_enabled'],
                                                     self.tribler_settings['download_defaults']['saveas'], [], 0)
 
         if self.dialog:
@@ -709,18 +797,32 @@ class TriblerWindow(QMainWindow):
             # This is necessary since VLC takes the screen and the popup becomes invisible.
             self.clicked_menu_button_home()
 
-        self.dialog = ConfirmationDialog(self, "Add torrent from URL/magnet link",
-                                         "Please enter the URL/magnet link in the field below:",
-                                         [('ADD', BUTTON_TYPE_NORMAL), ('CANCEL', BUTTON_TYPE_CONFIRM)],
-                                         show_input=True)
-        self.dialog.dialog_widget.dialog_input.setPlaceholderText('URL/magnet link')
-        self.dialog.dialog_widget.dialog_input.setFocus()
-        self.dialog.button_clicked.connect(self.on_torrent_from_url_dialog_done)
-        self.dialog.show()
+        if not self.add_torrent_url_dialog_active:
+            self.dialog = ConfirmationDialog(self, "Add torrent from URL/magnet link",
+                                             "Please enter the URL/magnet link in the field below:",
+                                             [('ADD', BUTTON_TYPE_NORMAL), ('CANCEL', BUTTON_TYPE_CONFIRM)],
+                                             show_input=True)
+            self.dialog.dialog_widget.dialog_input.setPlaceholderText('URL/magnet link')
+            self.dialog.dialog_widget.dialog_input.setFocus()
+            self.dialog.button_clicked.connect(self.on_torrent_from_url_dialog_done)
+            self.dialog.show()
+            self.add_torrent_url_dialog_active = True
 
     def on_torrent_from_url_dialog_done(self, action):
+        self.add_torrent_url_dialog_active = False
         if self.dialog and self.dialog.dialog_widget:
-            uri = self.dialog.dialog_widget.dialog_input.text()
+            uri = self.dialog.dialog_widget.dialog_input.text().strip()
+
+            # If the URI is a 40-bytes hex-encoded infohash, convert it to a valid magnet link
+            if len(uri) == 40:
+                valid_ih_hex = True
+                try:
+                    int(uri, 16)
+                except ValueError:
+                    valid_ih_hex = False
+
+                if valid_ih_hex:
+                    uri = "magnet:?xt=urn:btih:" + uri
 
             # Remove first dialog
             self.dialog.close_dialog()
@@ -771,6 +873,7 @@ class TriblerWindow(QMainWindow):
         self.deselect_all_menu_buttons(self.left_menu_button_discovered)
         self.stackedWidget.setCurrentIndex(PAGE_DISCOVERED)
         self.discovered_page.load_discovered_channels()
+        self.discovered_channels_list.setFocus()
         self.navigation_stack = []
         self.hide_left_menu_playlist()
 
@@ -787,23 +890,29 @@ class TriblerWindow(QMainWindow):
         self.navigation_stack = []
         self.show_left_menu_playlist()
 
+    def clicked_menu_button_trust_graph(self):
+        self.deselect_all_menu_buttons(self.left_menu_button_trust_graph)
+        self.stackedWidget.setCurrentIndex(PAGE_TRUST_GRAPH_PAGE)
+        self.navigation_stack = []
+        self.hide_left_menu_playlist()
+
     def clicked_menu_button_downloads(self):
+        self.deselect_all_menu_buttons(self.left_menu_button_downloads)
         self.raise_window()
         self.left_menu_button_downloads.setChecked(True)
-        self.deselect_all_menu_buttons(self.left_menu_button_downloads)
         self.stackedWidget.setCurrentIndex(PAGE_DOWNLOADS)
         self.navigation_stack = []
         self.hide_left_menu_playlist()
 
     def clicked_menu_button_debug(self):
         if not self.debug_window:
-            self.debug_window = DebugWindow(self.tribler_settings, self.core_manager.events_manager.tribler_version)
+            self.debug_window = DebugWindow(self.tribler_settings, self.tribler_version)
         self.debug_window.show()
 
     def clicked_menu_button_subscriptions(self):
         self.deselect_all_menu_buttons(self.left_menu_button_subscriptions)
-        self.subscribed_channels_page.load_subscribed_channels()
         self.stackedWidget.setCurrentIndex(PAGE_SUBSCRIBED_CHANNELS)
+        self.subscribed_channels_page.load_subscribed_channels()
         self.navigation_stack = []
         self.hide_left_menu_playlist()
 
@@ -817,34 +926,10 @@ class TriblerWindow(QMainWindow):
         self.left_menu_playlist_label.setHidden(False)
         self.left_menu_playlist.setHidden(False)
 
-    def on_channel_item_click(self, channel_list_item):
-        list_widget = channel_list_item.listWidget()
-        from TriblerGUI.widgets.channel_list_item import ChannelListItem
-        if isinstance(list_widget.itemWidget(channel_list_item), ChannelListItem):
-            channel_info = channel_list_item.data(Qt.UserRole)
-            self.channel_page.initialize_with_channel(channel_info)
-            self.navigation_stack.append(self.stackedWidget.currentIndex())
-            self.stackedWidget.setCurrentIndex(PAGE_CHANNEL_DETAILS)
-
-    def on_playlist_item_click(self, playlist_list_item):
-        list_widget = playlist_list_item.listWidget()
-        from TriblerGUI.widgets.playlist_list_item import PlaylistListItem
-        if isinstance(list_widget.itemWidget(playlist_list_item), PlaylistListItem):
-            playlist_info = playlist_list_item.data(Qt.UserRole)
-            self.playlist_page.initialize_with_playlist(playlist_info)
-            self.navigation_stack.append(self.stackedWidget.currentIndex())
-            self.stackedWidget.setCurrentIndex(PAGE_PLAYLIST_DETAILS)
-
     def on_page_back_clicked(self):
         try:
             prev_page = self.navigation_stack.pop()
             self.stackedWidget.setCurrentIndex(prev_page)
-            if prev_page == PAGE_SEARCH_RESULTS:
-                self.stackedWidget.widget(prev_page).load_search_results_in_list()
-            if prev_page == PAGE_SUBSCRIBED_CHANNELS:
-                self.stackedWidget.widget(prev_page).load_subscribed_channels()
-            if prev_page == PAGE_DISCOVERED:
-                self.stackedWidget.widget(prev_page).load_discovered_channels()
         except IndexError:
             logging.exception("Unknown page found in stack")
 
@@ -882,7 +967,6 @@ class TriblerWindow(QMainWindow):
             self.show_loading_screen()
             self.hide_status_bar()
             self.loading_text_label.setText("Shutting down...")
-
             if self.debug_window:
                 self.debug_window.setHidden(True)
 
@@ -942,10 +1026,29 @@ class TriblerWindow(QMainWindow):
         # Stop the Qt application
         QApplication.quit()
 
+    def clicked_skip_conversion(self):
+        self.dialog = ConfirmationDialog(self, "Abort the conversion of old channels",
+                                         "The upgrade procedure is now converting torrents in channels "
+                                         "collected by the previous installation of Tribler.\n\n"
+                                         "Are you sure you want to abort the conversion process?\n",
+                                         [('ABORT', BUTTON_TYPE_CONFIRM),
+                                          ('CONTINUE', BUTTON_TYPE_NORMAL)])
+        self.dialog.button_clicked.connect(self.on_skip_conversion_dialog)
+        self.dialog.show()
+
+    def on_skip_conversion_dialog(self, action):
+        if action == 0:
+            request_mgr = TriblerRequestManager()
+            request_mgr.perform_request("upgrader", lambda _: None, data={"skip_db_upgrade": True}, method='POST')
+
+        if self.dialog:
+            self.dialog.close_dialog()
+            self.dialog = None
+
     def on_tribler_shutdown_state_update(self, state):
         self.loading_text_label.setText(state)
 
 
 def _qurl_to_path(qurl):
-    parsed = urlparse.urlparse(qurl.toString())
+    parsed = urlparse(qurl.toString())
     return os.path.abspath(os.path.join(parsed.netloc, unquote(parsed.path)))
